﻿/**************************************************************************************
 * 
 * Name: ParallelPort
 *
 **************************************************************************************/
namespace ParallelPort
{
    using System;
    using System.IO;
    using System.Runtime.InteropServices;
    using Microsoft.Win32.SafeHandles;
    using System.IO.Ports;
    using System.Text;
    using System.Windows.Forms;

    //parallel port access contains four routines as follows:
    //write_lpt      - recieves a string command, converts to byte array and sends to lpt port
    //write_lpt_byte - recieves a byte array and sends to lpt port
    //read_lpt       - reads lpt port data into byte array, converts to a string and returns to parent
    //read_lpt_byte  - reads lpt port data into byte array, and returns to parent
    public class PortAccess
    {
        public const short FILE_ATTRIBUTE_NORMAL = 0x80;
        public const short INVALID_HANDLE_VALUE = -1;
        public const uint GENERIC_READ = 0x80000000;
        public const uint GENERIC_WRITE = 0x40000000;
        public const uint GENERIC_EXECUTE = 0x20000000;
        public const uint CREATE_NEW = 1;
        public const uint CREATE_ALWAYS = 2;
        public const uint OPEN_EXISTING = 3;

        //Createfile will return a safe file handle.  This can used with the filestream class.
        [DllImport("kernel32.dll", SetLastError = true)]
        static extern SafeFileHandle CreateFile(string lpFileName, uint dwDesiredAccess,
            uint dwShareMode, IntPtr lpSecurityAttributes, uint dwCreationDisposition,
            uint dwFlagsAndAttributes, IntPtr hTemplateFile);

        //This routine will test the parallel port selected for possible bidi communication
        public static bool test_lpt(string lptport)
        {
            bool status = false;
            int x;
            Byte[] buffer = new Byte[2];

            SafeFileHandle port = CreateFile(lptport, GENERIC_READ, 0, IntPtr.Zero, OPEN_EXISTING, 0, IntPtr.Zero);

            try
            {
                //Open port
                FileStream lpt = new FileStream(port, FileAccess.ReadWrite);

                //read one byte from port 
                x = lpt.Read(buffer, 0, 1);

                //Close port
                lpt.Close();
                if (x == 0)
                    MessageBox.Show("Printer Ready but no data avaiable now.");
                else
                    MessageBox.Show("Printer Ready.  The byte read is " + buffer[0].ToString());
                    
            }
            catch       //(Exception ex)
            {
                
                MessageBox.Show("Unable to read data from " + lptport + ".  Check/Set printer configuration items bidi parallel and status return.  Also verify the printer is on, that a parallel cable is plugged in and NO printer driver is installed on this parallel port.");
                //MessageBox.Show(ex.Message);
                //MessageBox.Show(ex.Message + ex.StackTrace);
                //MessageBox.Show(ex.ToString());
                
            }
            return (status);
        }
        public static bool write_lpt(string lptport, string command)
        {
            bool status = true;                             //set for success

            //use lpt port name to call create file and get safe file handle 
            SafeFileHandle port = CreateFile(lptport, GENERIC_WRITE, 0, IntPtr.Zero, OPEN_EXISTING, 0, IntPtr.Zero);


            try
            {
                //Open a file stream on LPT port by using safe file handle
                FileStream lpt = new FileStream(port, FileAccess.ReadWrite);

                Byte[] buffer = new Byte[command.Length];
                //Check to see if your printer support ASCII encoding or Unicode.
                //If unicode is supported, use the following:
                //buffer = System.Text.Encoding.Unicode.GetBytes(Temp);
                buffer = System.Text.Encoding.ASCII.GetBytes(command);
                lpt.Write(buffer, 0, buffer.Length);            //write data to lpt buffer
                lpt.Close();                                    //close port to flush buffer to printer
            }
            catch
            {
                MessageBox.Show("Unable to write to " + lptport);
                status = false;                                 //reset to failure
            }
            return (status);
        }

        public static bool write_lpt_byte(string lptport, Byte[] buffer)
        {
            bool status = true;                                 //set for success

            //use lpt port name to call create file and get safe file handle 
            SafeFileHandle port = CreateFile(lptport, GENERIC_WRITE, 0, IntPtr.Zero, OPEN_EXISTING, 0, IntPtr.Zero);

            try
            {
                //Open a file stream on LPT port by using safe file handle
                FileStream lpt = new FileStream(port, FileAccess.ReadWrite);
                lpt.Write(buffer, 0, buffer.Length);            //write data to lpt buffer
                lpt.Close();                                    //close port to flush buffer to printer
            }
            catch
            {
                MessageBox.Show("Unable to write to " + lptport);
                status = false;                                 //reset to failure
            }
            return (status);
        }

        public static string read_lpt(string lptport)
        {
            string temp = "";
            int x;
            Byte[] buffer = new Byte[1024];

            SafeFileHandle port = CreateFile(lptport, GENERIC_READ, 0, IntPtr.Zero, OPEN_EXISTING, 0, IntPtr.Zero);

            try
            {
                //Open port
                FileStream lpt = new FileStream(port, FileAccess.ReadWrite);
                //read from port into byte array 
                x = lpt.Read(buffer, 0, buffer.Length);
                
                //Close port
                lpt.Close();
                //If any data in byte array, convert to string
                if (x > 0)
                    temp = System.Text.Encoding.ASCII.GetString(buffer, 0, x);
                else
                    temp = "";
            }
            catch
            {
                MessageBox.Show("Unable to read data from " + lptport);
            }
            return (temp);
        }

        public static byte[] read_lpt_byte(string lptport)
        {
            int x;
            Byte[] buffer = new Byte[1024];

            SafeFileHandle port = CreateFile(lptport, GENERIC_READ, 0, IntPtr.Zero, OPEN_EXISTING, 0, IntPtr.Zero);

            try
            {
                //Open port
                FileStream lpt = new FileStream(port, FileAccess.ReadWrite);

                //read from port into byte array 
                x = lpt.Read(buffer, 0, buffer.Length);
                
                //Close port
                lpt.Close();
            }
            catch
            {
                MessageBox.Show("Unable to read data from " + lptport);
            }
            return (buffer);
        }

    }       //end class PortAccess

}           //end namespace ParallelPort