﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.IO;
using System.Threading;
using System.Windows.Forms;

namespace USBHID
{

    public class HIDDevice : IDisposable
    {
        bool disposed = false;
        public static bool data_available = false;
        //public static bool reading_flag = false;

        /*Variables --------------------------------------------------------------------*/
        private String vendorID;    //Vendor ID of the Device
        private String productID;   //Product ID of the Device
        private String devicePath;  //device path
        private int deviceCount;    //device count

        private bool connectionState;   //Connection Status true: connected, false: disconnected
        public int byteCount = 0;       //Recieved Bytes
        //USB Object
        private USBBoca myUSB = new USBBoca();
        
        //thread for read operations
        protected Thread dataReadingThread;
        public static HIDDevice boca_hid;
        private boca_sample.FrmBoca m_parent;

        public HIDDevice(boca_sample.FrmBoca frm1)
         {

            //create Read Thread
            dataReadingThread = new Thread(new ThreadStart(readDataThread));
            m_parent = (boca_sample.FrmBoca)frm1;
            boca_hid = this;

        }
         
        public static int Find_HIDs(int sn)
        {
            int x;

            x = boca_hid.Find_HID_Printers(sn);

            return (x);
        }
        
        public static int HID_Status()
        {
            //This routine checks the HID interface for status.  This is different from 
            //reading the printer status in the printer output buffer.
            int x;

            //There are four possible return values
            //HEX           DEC  Description
            //===========   ===  =========================
            //0x18        = 24   Printer Ready
            //0x20        = 32   Printer Out of Paper
            //0x10        = 16   General Printer Error
            //0xffffffff  - -1   No Response from printer
            x = boca_hid.get_HID_Status();
            return (x);
        }

 
         
        public string get_serial_number()
        {
            string sn = "";
            bool test;

            byte[] Buffer = new byte[128];

            //int myPtrToPreparsedData = -1;
            test = myUSB.CT_HidD_GetSerialNumberString(myUSB.HidHandle, ref Buffer[0], 128);
            if (test)
            {
                if ((Buffer[0] == 9) && (Buffer[1] == 4))
                    sn = "0000000000";              //HID mode reporting sn zero  2.1.0.0 see <dsn>, <esn> & <esn0>
                else
                    //sn = System.Text.Encoding.ASCII.GetString(Buffer);
                    sn = System.Text.Encoding.Unicode.GetString(Buffer);
            }
            else
                sn = "-1";
            return (sn);
        }
        

        public int get_HID_Status()
        {
                    
            bool success;
            byte[] Buffer = new byte[128];
            int i = -1;         //initialize to No Response

            success = HidApiDeclarations.HidD_GetInputReport(myUSB.HidWriteHandle, ref Buffer[0], 128);
            //if successful response then the report ID will be in Buffer[0] and the status value will be in Buffer[1]
            if (success)
                i = Buffer[1];

            return (i);
        }

         
        //the intent here is to perform the string writes by converting to byte arrays
        public static void write_HID(string command)
        {
            byte[] byte_array = new byte[command.Length];

            byte_array = System.Text.Encoding.ASCII.GetBytes(command);

            //perform the write
            boca_hid.writeData(byte_array);

        }


        //the intent here is to perform the write using byte arrays
        public static void write_HIDb(byte[] byte_array)
        {
            //perform the write
            boca_hid.writeData(byte_array);
        }

        public static void start_stop_read_thread()
        {
            //Start/Stop read data thread
            boca_hid.readData();

        }
        public static void display_read_state()
        {
            boca_hid.dis_readthread_state();
        }

        //perform read
        public static string read_HID()
        {
            string incoming = "";
            byte[] frame = new byte[33];
            string str1 = "";
            int x, i, j;
            //bool connect;                                     //@mdh test

            //connect = boca_hid.getConnectionState();          //@mdh test
            //incoming = boca_hid.getVendorID();                //@mdh test
            //incoming = boca_hid.getProductID();               //@mdh test

            //The data is read into the list known as USBHIDDRIVER.USBDevice.usbBuffer
            //Move data from list to byte array temp
            x = USBHID.USBDevice.usbBuffer.Count;             //Number of frames 33 bytes each
            //System.Windows.Forms.MessageBox.Show("# of frames = " + x.ToString());          //@mdh test
            if (x > 0)
            {
                lock (USBHID.USBDevice.usbBuffer.SyncRoot)
                {
                    for (i = 0; i < x; i++)
                    {
                        //read all data from each frame in the USB Buffer
                        frame = (byte[])USBHID.USBDevice.usbBuffer[i];
                        //k = frame.Length;                    
                        str1 += System.Text.Encoding.ASCII.GetString(frame);
                    }
                //now that all data has been copied, clear the incoming buffer for more possible data
                USBHID.USBDevice.usbBuffer.Clear();
                }

                j = 0;
                byte[] temp2 = new byte[str1.Length];
                //remove all nulls embedded in string
                for (i = 0; i < str1.Length; i++)
                {
                    if (str1[i] != '\0')
                    {
                        temp2[j] = (byte)str1[i];           //if something not null, save it
                        j++;                                //increment character counter
                    }
                }

                //incoming string is built here from byte array temp2 with j number of characters.
                incoming = System.Text.Encoding.ASCII.GetString(temp2, 0, j);
            }

            //return string as promised
            return (incoming);
        }

        public bool connectDevice()
        {
            //searchDevice
            searchDevice();
            //return connection state
            return this.getConnectionState();
        }

        //Scan/pick Boca printers
        private int Find_HID_Printers(int sn_flag)
        {
            //sn_flag will equal -1 when scanning for all Boca HID printer
            //sn_flag will contain an actual sn when picking a specific printer

            //no device found yet
            //bool deviceFound = false;
            bool lastdevice = false;
            this.deviceCount = 0;
            this.devicePath = string.Empty;
            string sn;
            //bool bocadevicedetected = false;
            int result = -1;
            int resultb = -1;
            int device_count = 0;
            int size = 0;
            int requiredSize = 0;
            int boca_count = 0;
            int snx;

            //HidD_GetHidGuid - get the GUID for all system HIDs
            myUSB.CT_HidGuid();

            //SetupDiGetClassDevs - returns handle to device information for all installed HIDs
            myUSB.CT_SetupDiGetClassDevs();

            USBBoca.security.lpSecurityDescriptor = 0;
            USBBoca.security.bInheritHandle = System.Convert.ToInt32(true);
            USBBoca.security.nLength = Marshal.SizeOf(USBBoca.security);

            //search for all Boca Systems HID mode printers and store information in printer table
            do
            {
                //open the device
                result = myUSB.CT_SetupDiEnumDeviceInterfaces(device_count);
                if (result == 0)
                    lastdevice = true;
                else
                {
                    //get size of device path
                    resultb = myUSB.CT_SetupDiGetDeviceInterfaceDetail(ref requiredSize, 0);
                    size = requiredSize;
                    //get device path
                    resultb = myUSB.CT_SetupDiGetDeviceInterfaceDetailx(ref requiredSize, size);

                    //looking for all Boca Systems USB printers in HID mode.
                    //Boca Systems USB vendor ID is 0x0a43.  The string below should be found in the 
                    //devicepathname if it is a Boca printer.
                    string deviceID = "vid_0a43";
                    if (myUSB.DevicePathName.IndexOf(deviceID) > 0)
                    {
                        //store device information
                        this.deviceCount = device_count;
                        this.devicePath = myUSB.DevicePathName;

                        //init device
                        myUSB.CT_SetupDiEnumDeviceInterfaces(this.deviceCount);

                        size = 0;
                        requiredSize = 0;

                        resultb = myUSB.CT_SetupDiGetDeviceInterfaceDetail(ref requiredSize, size);
                        resultb = 0;
                        resultb = myUSB.CT_SetupDiGetDeviceInterfaceDetailx(ref requiredSize, size);
                        resultb = 0;

                        //create HID Device Handle to open a channel for reading product id and serial number
                        resultb = myUSB.CT_CreateFile(this.devicePath);

                        if (resultb != 0)
                        {
                            result = myUSB.CT_HidD_GetAttributes(myUSB.HidHandle);
                            //At the time this was written only FGL44 and FGL46 printers can be put into HID mode
                            if (myUSB.myHIDD_ATTRIBUTES.VendorID == boca_sample.FrmBoca.BOCA_VID)
                            {
                                sn = get_serial_number();           //get serial number string
                                snx = Convert.ToInt32(sn);          //convert to integer

                                //sn_flag is used to control one of two functions performed here
                                //when sn_flag is -1 then scan PC for all Boca HIDs and store information in table for use later
                                //otherwise sn_flag will contain the serial number of a specific Boca HID to select now.
                                if (sn_flag == -1)
                                {
                                    boca_sample.FrmBoca.Printer_Table[boca_count].Handle = myUSB.HidHandle;
                                    boca_sample.FrmBoca.Printer_Table[boca_count].SerialNumber = snx;
                                    boca_sample.FrmBoca.Printer_Table[boca_count].ProductID = myUSB.myHIDD_ATTRIBUTES.ProductID;
                                    boca_count++;
                                    myUSB.CT_CloseHandle(myUSB.HidHandle);          //close port
                                }
                                else
                                {
                                    //if the serial numbers match leave the port open and exit the loop
                                    if (snx == sn_flag)
                                    {
                                        lastdevice = true;
                                        boca_count = 1;
                                        
                                        //Save the open channel handle as the write handle
                                        myUSB.HidWriteHandle = myUSB.HidHandle;                         
                                        
                                        //create HID Device Read Handle to open a channel for reading status
                                        resultb = myUSB.CT_CreateFile(this.devicePath);      
                                        if (resultb == 1)
                                        {
                                            //Save the open channel handle as the read handle
                                            myUSB.HidReadHandle = myUSB.HidHandle;                  
                                            myUSB.HidHandle = -1;                                   
                                            myUSB.prepareforoverlappedtransfer();                   
                                            setConnectionState(true);                               
                                            start_stop_read_thread();                               //start read thread
                                        }
                                    }
                                    else
                                    {
                                        myUSB.CT_CloseHandle(myUSB.HidHandle);      //close port
                                        boca_count = -1;
                                    }
                                }
                            }
                        }
                    }

                    device_count++;
                }
            } while (lastdevice == false);

            return (boca_count);
        }
        private bool searchDevice()
        {
            //no device found yet
            bool deviceFound = false;
            this.deviceCount = 0;
            this.devicePath = string.Empty;

            myUSB.CT_HidGuid();
            myUSB.CT_SetupDiGetClassDevs();

            int result = -1;
            int resultb = -1;
            int device_count = 0;
            int size = 0;
            int requiredSize = 0;

            //search the device until you have found it or no more devices in list
            while (result != 0)
            {
                //open the device
                result = myUSB.CT_SetupDiEnumDeviceInterfaces(device_count);
                //get size of device path
                resultb = myUSB.CT_SetupDiGetDeviceInterfaceDetail(ref requiredSize, 0);
                size = requiredSize;
                //get device path
                resultb = myUSB.CT_SetupDiGetDeviceInterfaceDetailx(ref requiredSize, size);

                //is this the device i want?
                //string deviceID = this.vendorID + "&" + this.productID;
                string deviceID = this.vendorID;
                if (myUSB.DevicePathName.IndexOf(deviceID) > 0)
                {
                    //yes it is

                    //store device information
                    this.deviceCount = device_count;
                    this.devicePath = myUSB.DevicePathName;
                    deviceFound = true;

                    //init device
                    myUSB.CT_SetupDiEnumDeviceInterfaces(this.deviceCount);

                    size = 0;
                    requiredSize = 0;

                    resultb = myUSB.CT_SetupDiGetDeviceInterfaceDetail(ref requiredSize, size);
                    resultb = 0;
                    resultb = myUSB.CT_SetupDiGetDeviceInterfaceDetailx(ref requiredSize, size);
                    resultb = 0;
                    //create HID Device Handle
                    resultb = myUSB.CT_CreateFile(this.devicePath);

                    //we have found our device so stop searching
                    break;
                }
                device_count++;
            }

            //set connection state
            this.setConnectionState(deviceFound);
            //return state
            return this.getConnectionState();
        }

        public int getDevice()
        {

            this.deviceCount = 0;
            this.devicePath = string.Empty;

            myUSB.CT_HidGuid();
            myUSB.CT_SetupDiGetClassDevs();

            int result = -1;
            int resultb = -1;
            int device_count = 0;
            int size = 0;
            int requiredSize = 0;
            int numberOfDevices = 0;
            //search the device until you have found it or no more devices in list
            while (result != 0)
            {
                //open the device
                result = myUSB.CT_SetupDiEnumDeviceInterfaces(device_count);
                //get size of device path
                resultb = myUSB.CT_SetupDiGetDeviceInterfaceDetail(ref requiredSize, 0);
                size = requiredSize;
                //get device path
                resultb = myUSB.CT_SetupDiGetDeviceInterfaceDetailx(ref requiredSize, size);

                //is this the device i want?
                //string deviceID = this.vendorID + "&" + this.productID;
                string deviceID = this.vendorID;
                if (myUSB.DevicePathName.IndexOf(deviceID) > 0)
                {
                    numberOfDevices++;
                }
                device_count++;
            }
            return numberOfDevices;
        }

        public bool writeData(byte[] bDataToWrite)
        {
            bool success = false;

            if (getConnectionState())
            {
                try
                {
                    //set output report length
                    int outputReportByteLength = 33;

                    int bytesSend = 0;
                    //if bWriteData is bigger then one report diveide into sevral reports
                    while (bytesSend < bDataToWrite.Length)
                    {

                        // Set the size of the Output report buffer.
                        byte[] OutputReportBuffer = new byte[outputReportByteLength - 1 + 1];
                        // Store the report ID in the first byte of the buffer:
                        OutputReportBuffer[0] = 0;

                        // Store the report data following the report ID.
                        for (int i = 1; i < OutputReportBuffer.Length; i++)
                        {
                            if (bytesSend < bDataToWrite.Length)
                            {
                                OutputReportBuffer[i] = bDataToWrite[bytesSend];
                                bytesSend++;
                            }
                            else
                            {
                                OutputReportBuffer[i] = 0;
                            }
                        }

                        OutputReport myOutputReport = new OutputReport();
                        success = myOutputReport.Write(OutputReportBuffer, myUSB.HidWriteHandle);

                    }
                }
                catch (System.AccessViolationException)
                {
                    success = false;
                }
            }
            else
            {
                success = false;
            }
            return success;
        }

        public void readDataThread()
        {
            int receivedNull = 0;
            //string incoming = "";
            //bool call_flag = false;

            //while (true)
            while (boca_sample.FrmBoca.checking_for_data == true)
            {
                int myPtrToPreparsedData = -1;
                if (myUSB.CT_HidD_GetPreparsedData(myUSB.HidReadHandle, ref myPtrToPreparsedData) != 0)
                {
                    int code = myUSB.CT_HidP_GetCaps(myPtrToPreparsedData);
                    int reportLength = myUSB.myHIDP_CAPS.InputReportByteLength;

                    //while (true)
                    while (boca_sample.FrmBoca.checking_for_data == true)
                    {
                        //read until thread is stopped
                        byte[] myRead = myUSB.CT_ReadFile(myUSB.myHIDP_CAPS.InputReportByteLength);
                        if (myRead != null)
                        {
                            //reading_flag = true;
                            //ByteCount + bytes received
                            byteCount += myRead.Length;
                            //Store received bytes
                            lock (USBHID.USBDevice.usbBuffer.SyncRoot)
                            {
                                USBHID.USBDevice.usbBuffer.Add(myRead);
                                data_available = true;
                            }

                        }
                        else
                        {
                            //reading_flag = false;
                            //Recieved a lot of null bytes!
                            //maybe device disconnected?
                            if (receivedNull > 100)
                            {
                                receivedNull = 0;
                                Thread.Sleep(1);
                            }
                            receivedNull++;
                        }

                            if (data_available)
                            {
                            //Allow readthread to process  data and send it back to FrmBoca to display
                            if (myRead.Length > 0)
                            {
                                boca_hid.m_parent.incoming += read_HID();                   //perform read
 
                                if (boca_hid.m_parent.incoming.Length > 0)                  //if something in the buffer
                                    boca_hid.m_parent.Establish_Status();                   //display it

                                data_available = false;
                            }
                        }

                    }
                }
            }
        }

        public void dis_readthread_state()
        {
            //m_parent.Display_Text("Read Thread State = " + dataReadingThread.ThreadState.ToString());
            //m_parent.Display_Text("Read Thread State = " + boca_hid.dataReadingThread.ThreadState.ToString());
            MessageBox.Show("Read Thread State = " + dataReadingThread.ThreadState.ToString());
        }
        public void readData()
        {

            if (dataReadingThread.ThreadState.ToString() == "Unstarted")
            {
                //start the thread
                dataReadingThread.Start();
                Thread.Sleep(0);
            }
            else if (dataReadingThread.ThreadState.ToString() == "Running")
            {
                boca_sample.FrmBoca.checking_for_data = false;
                //Stop the Thread
                dataReadingThread.Abort();
                Thread.Sleep(0);
            }
            else
            {
                //create Read Thread
                dataReadingThread = new Thread(new ThreadStart(readDataThread));
                Thread.Sleep(100);

                boca_sample.FrmBoca.checking_for_data = true;

                //start the thread
                dataReadingThread.Start();
                Thread.Sleep(100);

            }

        }

         public void abortreadData()
        {

            if (dataReadingThread.ThreadState.ToString() == "Running")
            {
                //Stop the Thread
                dataReadingThread.Abort();
            }

        }

        public void disconnectDevice()
        {

            //close the read and write channels
            if (myUSB.HidWriteHandle != -1)
                USBBoca.CloseHandle(myUSB.HidWriteHandle);
            if (myUSB.HidReadHandle != -1)
                USBBoca.CloseHandle(myUSB.HidReadHandle);

            //If the thread is running, stop it.
            if (dataReadingThread.ThreadState.ToString() == "Running")
            {
                dataReadingThread.Abort();
                Thread.Sleep(100);
            }

            //set state to disconnected
            setConnectionState(false);

        }

        public void setDeviceData(String vID, String pID)
        {
            this.vendorID = vID;
            this.productID = pID;
        }

        public String getVendorID()
        {
            return this.vendorID;
        }
        public String getProductID()
        {
            return this.productID;
        }
        public void setConnectionState(bool state)
        {
            this.connectionState = state;
        }
        public bool getConnectionState()
        {
            return this.connectionState;
        }
        public List<string> getDevices()
        {
            List<string> devices = new List<string>();

            //no device found yet
            // bool deviceFound = false;
            this.deviceCount = 0;
            this.devicePath = string.Empty;

            myUSB.CT_HidGuid();
            myUSB.CT_SetupDiGetClassDevs();

            int result = -1;
            int resultb = -1;
            int device_count = 0;
            int size = 0;
            int requiredSize = 0;
            int numberOfDevices = 0;
            //search the device until you have found it or no more devices in list

            while (result != 0)
            {
                //open the device
                result = myUSB.CT_SetupDiEnumDeviceInterfaces(device_count);
                //get size of device path
                resultb = myUSB.CT_SetupDiGetDeviceInterfaceDetail(ref requiredSize, 0);
                size = requiredSize;
                //get device path
                resultb = myUSB.CT_SetupDiGetDeviceInterfaceDetailx(ref requiredSize, size);

                //is this the device i want?
                string deviceID = this.vendorID;
                if (myUSB.DevicePathName.IndexOf(deviceID) > 0)
                {
                    devices.Add(myUSB.DevicePathName);
                    numberOfDevices++;
                }
                device_count++;
            }
            return devices;
        }
        public string getDevicePath()
        {
            return this.devicePath;
        }

        internal abstract class HostReport
        {

            // For reports the host sends to the device.
            // Each report class defines a ProtectedWrite method for writing a type of report.
            protected abstract bool ProtectedWrite(int deviceHandle, byte[] reportBuffer);
            internal bool Write(byte[] reportBuffer, int deviceHandle)
            {

                bool Success = false;

                // Purpose    : Calls the overridden ProtectedWrite routine.
                //            : This method enables other classes to override ProtectedWrite
                //            : while limiting access as Friend.
                //            : (Directly declaring Write as Friend MustOverride causes the
                //            : compiler(warning) "Other languages may permit Friend
                //            : Overridable members to be overridden.")
                // Accepts    : reportBuffer - contains the report ID and report data.
                //            : deviceHandle - handle to the device.             '
                // Returns    : True on success. False on failure.

                try
                {
                    Success = ProtectedWrite(deviceHandle, reportBuffer);
                }
                catch (Exception)
                {

                }

                return Success;
            }
        }



        internal class OutputReport : HostReport
        {
            protected override bool ProtectedWrite(int hidHandle, byte[] outputReportBuffer)
            {

                // Purpose    : writes an Output report to the device.
                // Accepts    : HIDHandle - a handle to the device.
                //              OutputReportBuffer - contains the report ID and report to send.
                // Returns    : True on success. False on failure.

                int NumberOfBytesWritten = 0;
                int Result;
                bool Success = false;
                //int result = 0;
                //bool success = false;
                //int count;

                try
                {
                    // The host will use an interrupt transfer if the the HID has an interrupt OUT
                    // endpoint (requires USB 1.1 or later) AND the OS is NOT Windows 98 Gold (original version).
                    // Otherwise the the host will use a control transfer.
                    // The application doesn't have to know or care which type of transfer is used.

                    // ***
                    // API function: WriteFile
                    // Purpose: writes an Output report to the device.
                    // Accepts:
                    // A handle returned by CreateFile
                    // The output report byte length returned by HidP_GetCaps.
                    // An integer to hold the number of bytes written.
                    // Returns: True on success, False on failure.
                    // ***

                    Result = USBBoca.WriteFile(hidHandle, ref outputReportBuffer[0], outputReportBuffer.Length, ref NumberOfBytesWritten, 0);  //@mdh overlap
                    Success = (Result == 0) ? false : true;

                }
                catch (Exception)
                {
                }

                return Success;
            }


        }

        #region IDisposable Members

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected void Dispose(bool disposeManagedResources)
        {
            if (!this.disposed)
            {
                if (disposeManagedResources)
                {
                    //only clear up managed stuff here
                }

                //clear up unmanaged stuff here
                if (myUSB.HidHandle != -1)
                {
                    myUSB.CT_CloseHandle(myUSB.HidHandle);
                }
                //clear up unmanaged stuff here
                if (myUSB.HidReadHandle != -1)
                {
                    myUSB.CT_CloseHandle(myUSB.HidReadHandle);
                }
                //clear up unmanaged stuff here
                if (myUSB.HidWriteHandle != -1)
                {
                    myUSB.CT_CloseHandle(myUSB.HidWriteHandle);
                }

                if (myUSB.hDevInfo != -1)
                {
                    myUSB.CT_SetupDiDestroyDeviceInfoList();
                }

                this.disposed = true;
            }
        }

        #endregion

    }

    public class USBBoca
    {

        #region Structs and DLL-Imports
        // Required constants, pointers, handles and variables 
        public int HidHandle = -1;				//file handle for a Hid devices
        public int HidWriteHandle = -1;			//Write file handle for a Hid devices
        public int HidReadHandle = -1;			//Read file handle for a Hid devices
        public int hDevInfo = -1;				// handle for the device infoset
        public string DevicePathName = "";
        public const int DIGCF_PRESENT = 0x00000002;
        public const int DIGCF_DEVICEINTERFACE = 0x00000010;
        public const int DIGCF_INTERFACEDEVICE = 0x00000010;
        public const uint GENERIC_READ = 0x80000000;
        public const uint GENERIC_WRITE = 0x40000000;
        public const uint FILE_SHARE_READ = 0x00000001;
        public const uint FILE_SHARE_WRITE = 0x00000002;
        public const int OPEN_EXISTING = 3;
        public const int EV_RXFLAG = 0x0002;    // received certain character

        // specified in DCB
        public const int INVALID_HANDLE_VALUE = -1;
        public const int ERROR_INVALID_HANDLE = 6;
        public const int FILE_FLAG_OVERLAPED = 0x40000000;

        // GUID structure
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct GUID
        {
            public int Data1;
            public System.UInt16 Data2;
            public System.UInt16 Data3;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
            public byte[] data4;
        }

        // Device interface data
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct SP_DEVICE_INTERFACE_DATA
        {
            public int cbSize;
            public GUID InterfaceClassGuid;
            public int Flags;
            public int Reserved;
        }

        // Device interface detail data
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public unsafe struct PSP_DEVICE_INTERFACE_DETAIL_DATA
        {
            public int cbSize;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
            public string DevicePath;
        }

        // HIDD_ATTRIBUTES
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct HIDD_ATTRIBUTES
        {
            public int Size; // = sizeof (struct _HIDD_ATTRIBUTES) = 10

            //
            // Vendor ids of this hid device
            //
            public System.UInt16 VendorID;
            public System.UInt16 ProductID;
            public System.UInt16 VersionNumber;

            //
            // Additional fields will be added to the end of this structure.
            //
        }

        // HIDP_CAPS
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct HIDP_CAPS
        {
            public System.UInt16 Usage;					// USHORT
            public System.UInt16 UsagePage;				// USHORT
            public System.UInt16 InputReportByteLength;
            public System.UInt16 OutputReportByteLength;
            public System.UInt16 FeatureReportByteLength;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 17)]
            public System.UInt16[] Reserved;				// USHORT  Reserved[17];			
            public System.UInt16 NumberLinkCollectionNodes;
            public System.UInt16 NumberInputButtonCaps;
            public System.UInt16 NumberInputValueCaps;
            public System.UInt16 NumberInputDataIndices;
            public System.UInt16 NumberOutputButtonCaps;
            public System.UInt16 NumberOutputValueCaps;
            public System.UInt16 NumberOutputDataIndices;
            public System.UInt16 NumberFeatureButtonCaps;
            public System.UInt16 NumberFeatureValueCaps;
            public System.UInt16 NumberFeatureDataIndices;
        }

        //HIDP_REPORT_TYPE 
        public enum HIDP_REPORT_TYPE
        {
            HidP_Input,		// 0 input
            HidP_Output,	// 1 output
            HidP_Feature	// 2 feature
        }

        // Structures in the union belonging to HIDP_VALUE_CAPS (see below)

        // Range
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct Range
        {
            public System.UInt16 UsageMin;			// USAGE	UsageMin; // USAGE  Usage; 
            public System.UInt16 UsageMax; 			// USAGE	UsageMax; // USAGE	Reserved1;
            public System.UInt16 StringMin;			// USHORT  StringMin; // StringIndex; 
            public System.UInt16 StringMax;			// USHORT	StringMax;// Reserved2;
            public System.UInt16 DesignatorMin;		// USHORT  DesignatorMin; // DesignatorIndex; 
            public System.UInt16 DesignatorMax;		// USHORT	DesignatorMax; //Reserved3; 
            public System.UInt16 DataIndexMin;		// USHORT  DataIndexMin;  // DataIndex; 
            public System.UInt16 DataIndexMax;		// USHORT	DataIndexMax; // Reserved4;
        }

        // Range
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct NotRange
        {
            public System.UInt16 Usage;
            public System.UInt16 Reserved1;
            public System.UInt16 StringIndex;
            public System.UInt16 Reserved2;
            public System.UInt16 DesignatorIndex;
            public System.UInt16 Reserved3;
            public System.UInt16 DataIndex;
            public System.UInt16 Reserved4;
        }

        //HIDP_VALUE_CAPS
        [StructLayout(LayoutKind.Explicit, CharSet = CharSet.Ansi)]
        public unsafe struct HIDP_VALUE_CAPS
        {
            //
            [FieldOffset(0)]
            public System.UInt16 UsagePage;					    // USHORT
            [FieldOffset(2)]
            public System.Byte ReportID;						// UCHAR  ReportID;
            [MarshalAs(UnmanagedType.I1)]
            [FieldOffset(3)]
            public System.Boolean IsAlias;						// BOOLEAN  IsAlias;
            [FieldOffset(4)]
            public System.UInt16 BitField;						// USHORT  BitField;
            [FieldOffset(6)]
            public System.UInt16 LinkCollection;				//USHORT  LinkCollection;
            [FieldOffset(8)]
            public System.UInt16 LinkUsage;					    // USAGE  LinkUsage;
            [FieldOffset(10)]
            public System.UInt16 LinkUsagePage;				    // USAGE  LinkUsagePage;
            [MarshalAs(UnmanagedType.I1)]
            [FieldOffset(12)]
            public System.Boolean IsRange;					    // BOOLEAN  IsRange;
            [MarshalAs(UnmanagedType.I1)]
            [FieldOffset(13)]
            public System.Boolean IsStringRange;				// BOOLEAN  IsStringRange;
            [MarshalAs(UnmanagedType.I1)]
            [FieldOffset(14)]
            public System.Boolean IsDesignatorRange;			// BOOLEAN  IsDesignatorRange;
            [MarshalAs(UnmanagedType.I1)]
            [FieldOffset(15)]
            public System.Boolean IsAbsolute;					// BOOLEAN  IsAbsolute;
            [MarshalAs(UnmanagedType.I1)]
            [FieldOffset(16)]
            public System.Boolean HasNull;					    // BOOLEAN  HasNull;
            [FieldOffset(17)]
            public System.Char Reserved;						// UCHAR  Reserved;
            [FieldOffset(18)]
            public System.UInt16 BitSize;						// USHORT  BitSize;
            [FieldOffset(20)]
            public System.UInt16 ReportCount;					// USHORT  ReportCount;
            [FieldOffset(22)]
            public System.UInt16 Reserved2a;					// USHORT  Reserved2[5];		
            [FieldOffset(24)]
            public System.UInt16 Reserved2b;					// USHORT  Reserved2[5];
            [FieldOffset(26)]
            public System.UInt16 Reserved2c;					// USHORT  Reserved2[5];
            [FieldOffset(28)]
            public System.UInt16 Reserved2d;					// USHORT  Reserved2[5];
            [FieldOffset(30)]
            public System.UInt16 Reserved2e;					// USHORT  Reserved2[5];
            [FieldOffset(32)]
            public System.UInt16 UnitsExp;					    // ULONG  UnitsExp;
            [FieldOffset(34)]
            public System.UInt16 Units;						    // ULONG  Units;
            [FieldOffset(36)]
            public System.Int16 LogicalMin;					    // LONG  LogicalMin;   ;
            [FieldOffset(38)]
            public System.Int16 LogicalMax;					    // LONG  LogicalMax
            [FieldOffset(40)]
            public System.Int16 PhysicalMin;					// LONG  PhysicalMin, 
            [FieldOffset(42)]
            public System.Int16 PhysicalMax;					// LONG  PhysicalMax;
            // The Structs in the Union			
            [FieldOffset(44)]
            public Range Range;
            [FieldOffset(44)]
            public Range NotRange;
        }


        // ----------------------------------------------------------------------------------
        //
        //
        //
        // 
        // Define istances of the structures
        //
        //

        private GUID MYguid = new GUID();
        //
        // SP_DEVICE_INTERFACE_DATA  mySP_DEVICE_INTERFACE_DATA = new SP_DEVICE_INTERFACE_DATA();
        //
        public SP_DEVICE_INTERFACE_DATA mySP_DEVICE_INTERFACE_DATA;
        //
        public PSP_DEVICE_INTERFACE_DETAIL_DATA myPSP_DEVICE_INTERFACE_DETAIL_DATA;
        // 
        public HIDD_ATTRIBUTES myHIDD_ATTRIBUTES;
        //
        public HIDP_CAPS myHIDP_CAPS;
        //
        public HIDP_VALUE_CAPS[] myHIDP_VALUE_CAPS;

        public int eventobject;
        public static SECURITY_ATTRIBUTES security;

        // ******************************************************************************
        // DLL Calls
        // ******************************************************************************

        //Get GUID for the HID Class
        [DllImport("hid.dll", SetLastError = true)]
        static extern unsafe void HidD_GetHidGuid(
            ref GUID lpHidGuid);

        //Get array of structures with the HID info
        [DllImport("setupapi.dll", SetLastError = true)]
        static extern unsafe int SetupDiGetClassDevs(
            ref GUID lpHidGuid,
            int* Enumerator,
            int* hwndParent,
            int Flags);


        //Get context structure for a device interface element
        //
        //  SetupDiEnumDeviceInterfaces returns a context structure for a device 
        //  interface element of a device information set. Each call returns information 
        //  about one device interface; the function can be called repeatedly to get information 
        //  about several interfaces exposed by one or more devices.
        //
        [DllImport("setupapi.dll", SetLastError = true)]
        static extern unsafe int SetupDiEnumDeviceInterfaces(
            int DeviceInfoSet,
            int DeviceInfoData,
            ref  GUID lpHidGuid,
            int MemberIndex,
            ref  SP_DEVICE_INTERFACE_DATA lpDeviceInterfaceData);


        //	Get device Path name
        //  Works for the first pass  --> to get the required size

        [DllImport("setupapi.dll", SetLastError = true)]
        static extern unsafe int SetupDiGetDeviceInterfaceDetail(
            int DeviceInfoSet,
            ref SP_DEVICE_INTERFACE_DATA lpDeviceInterfaceData,
            int* aPtr,
            int detailSize,
            ref int requiredSize,
            int* bPtr);

        //	Get device Path name
        //  Works for second pass (overide), once size value is known
        [DllImport("setupapi.dll", SetLastError = true)]
        static extern unsafe int SetupDiGetDeviceInterfaceDetail(
            int DeviceInfoSet,
            ref SP_DEVICE_INTERFACE_DATA lpDeviceInterfaceData,
            ref PSP_DEVICE_INTERFACE_DETAIL_DATA myPSP_DEVICE_INTERFACE_DETAIL_DATA,
            int detailSize,
            ref int requiredSize,
            int* bPtr);

        [DllImport("kernel32.dll", SetLastError = true)]
        static extern bool GetOverlappedResult(
            int hObject,								// IN HANDLE  HidDeviceObject,
            [In] ref System.Threading.NativeOverlapped lpOverlapped,
            out uint lpNumberOfBytesTransferred,
            bool bWait);

        // Get Create File
        [DllImport("kernel32.dll", SetLastError = true)]
        private static extern int CreateFile(
            string lpFileName,							// file name
            uint dwDesiredAccess,						// access mode
            uint dwShareMode,							// share mode
            uint lpSecurityAttributes,					// SD
            uint dwCreationDisposition,					// how to create
            uint dwFlagsAndAttributes,					// file attributes
            uint hTemplateFile							// handle to template file
            );


        [DllImport("hid.dll", SetLastError = true)]
        private static extern int HidD_GetAttributes(
            int hObject,								// IN HANDLE  HidDeviceObject,
            ref HIDD_ATTRIBUTES Attributes);			// OUT PHIDD_ATTRIBUTES  Attributes


        [DllImport("hid.dll", SetLastError = true)]
        private unsafe static extern int HidD_GetPreparsedData(
            int hObject,								// IN HANDLE  HidDeviceObject,
            ref int pPHIDP_PREPARSED_DATA);				// OUT PHIDP_PREPARSED_DATA  *PreparsedData


        [DllImport("hid.dll", SetLastError = true)]
        private unsafe static extern int HidP_GetCaps(
            int pPHIDP_PREPARSED_DATA,					// IN PHIDP_PREPARSED_DATA  PreparsedData,
            ref HIDP_CAPS myPHIDP_CAPS);				// OUT PHIDP_CAPS  Capabilities

        [DllImport("hid.dll")]
        public unsafe static extern bool HidD_SetOutputReport(int HidDeviceObject, ref byte lpReportBuffer, int ReportBufferLength);

        [DllImport("hid.dll")]
        public unsafe static extern bool HidD_GetSerialNumberString(int HidDeviceObject, ref byte lpReportBuffer, int ReportBufferLength);

        [DllImport("hid.dll", SetLastError = true)]
        private unsafe static extern int HidP_GetValueCaps(
            HIDP_REPORT_TYPE ReportType,								// IN HIDP_REPORT_TYPE  ReportType,
            [In, Out] HIDP_VALUE_CAPS[] ValueCaps,						// OUT PHIDP_VALUE_CAPS  ValueCaps,
            ref int ValueCapsLength,									// IN OUT PULONG  ValueCapsLength,
            int pPHIDP_PREPARSED_DATA);									// IN PHIDP_PREPARSED_DATA  PreparsedData

        [DllImport("setupapi.dll", SetLastError = true)]
        static extern unsafe int SetupDiDestroyDeviceInfoList(
            int DeviceInfoSet				// IN HDEVINFO  DeviceInfoSet
            );

        // 13
        [DllImport("hid.dll", SetLastError = true)]
        static extern unsafe int HidD_FreePreparsedData(
            int pPHIDP_PREPARSED_DATA			// IN PHIDP_PREPARSED_DATA  PreparsedData
            );



        // API declarations relating to file I/O.
        // ******************************************************************************
        // API constants
        // ******************************************************************************
        public const uint FILE_FLAG_OVERLAPPED = 0x40000000;
        public const int WAIT_TIMEOUT = 0x102;
        public const short WAIT_OBJECT_0 = 0;
        public const int WRITE_TIMEOUT_DEFAULT = 2000;          //2 seconds
        public const int READ_TIMEOUT_DEFAULT = 500;            //0.5 seconds

        // ******************************************************************************
        // Structures and classes for API calls, listed alphabetically
        // ******************************************************************************

        [StructLayout(LayoutKind.Sequential)]
        public struct OVERLAPPED
        {
            public int Internal;
            public int InternalHigh;
            public int Offset;
            public int OffsetHigh;
            public int hEvent;
        }

        public static System.Threading.NativeOverlapped HIDOverlapped;

        [StructLayout(LayoutKind.Sequential)]
        public struct SECURITY_ATTRIBUTES
        {
            public int nLength;
            public int lpSecurityDescriptor;
            public int bInheritHandle;
        }

        [DllImport("kernel32.dll")]
        static public extern int CancelIo(int hFile);

        [DllImport("kernel32.dll")]
        static public extern int CloseHandle(int hObject);

        [DllImport("kernel32.dll", CharSet = CharSet.Auto)]
        static public extern int CreateEvent(ref SECURITY_ATTRIBUTES SecurityAttributes, int bManualReset, int bInitialState, string lpName);

        [DllImport("kernel32.dll", CharSet = CharSet.Auto)]
        static public extern int
          CreateFile(string lpFileName, uint dwDesiredAccess, uint dwShareMode, ref SECURITY_ATTRIBUTES lpSecurityAttributes, int dwCreationDisposition, uint dwFlagsAndAttributes, int hTemplateFile);
        
        [DllImport("kernel32.dll")]
        static public extern int ReadFile(int hFile, byte[] lpBuffer, int nNumberOfBytesToRead,
            ref int lpNumberOfBytesRead, ref System.Threading.NativeOverlapped lpOverlapped);

        [DllImport("kernel32.dll")]
        static public extern int WaitForSingleObject(int hHandle, int dwMilliseconds);

        [DllImport("kernel32.dll")]
        static public extern int WriteFile(int hFile, ref byte lpBuffer, int nNumberOfBytesToWrite, ref int lpNumberOfBytesWritten, int lpOverlapped);



        #endregion

        //  CT_HidGuid()
        public unsafe void CT_HidGuid()
        {
            HidD_GetHidGuid(ref MYguid);	// 
        }

        //  CT_SetupDiGetClassDevs()
        public unsafe int CT_SetupDiGetClassDevs()
        {
            hDevInfo = SetupDiGetClassDevs(
                ref MYguid,
                null,
                null,
                DIGCF_INTERFACEDEVICE | DIGCF_PRESENT);
            return hDevInfo;
        }

        //  int CT_SetupDiEnumDeviceInterfaces(int memberIndex)
        public unsafe int CT_SetupDiEnumDeviceInterfaces(int memberIndex)
        {
            mySP_DEVICE_INTERFACE_DATA = new SP_DEVICE_INTERFACE_DATA();
            mySP_DEVICE_INTERFACE_DATA.cbSize = Marshal.SizeOf(mySP_DEVICE_INTERFACE_DATA);
            int result = SetupDiEnumDeviceInterfaces(
                hDevInfo,
                0,
                ref  MYguid,
                memberIndex,
                ref mySP_DEVICE_INTERFACE_DATA);
            return result;
        }


        //  int CT_SetupDiGetDeviceInterfaceDetail(ref int RequiredSize, int DeviceInterfaceDetailDataSize)
        public unsafe int CT_SetupDiGetDeviceInterfaceDetail(ref int RequiredSize, int DeviceInterfaceDetailDataSize)
        {
            int results =
            SetupDiGetDeviceInterfaceDetail(
                hDevInfo,							// IN HDEVINFO  DeviceInfoSet,
                ref mySP_DEVICE_INTERFACE_DATA,		// IN PSP_DEVICE_INTERFACE_DATA  DeviceInterfaceData,
                null,								// OUT PSP_DEVICE_INTERFACE_DETAIL_DATA  DeviceInterfaceDetailData,  OPTIONAL
                DeviceInterfaceDetailDataSize,		// IN DWORD  DeviceInterfaceDetailDataSize,
                ref RequiredSize,					// OUT PDWORD  RequiredSize,  OPTIONAL
                null); // 
            return results;
        }

        //  int CT_SetupDiEnumDeviceInterfaces(int memberIndex)
        public unsafe int CT_SetupDiGetDeviceInterfaceDetailx(ref int RequiredSize, int DeviceInterfaceDetailDataSize)
        {
            myPSP_DEVICE_INTERFACE_DETAIL_DATA = new PSP_DEVICE_INTERFACE_DETAIL_DATA();

            //
            // This part needs some work

            // if I use the following line of code 
            // myPSP_DEVICE_INTERFACE_DETAIL_DATA.cbSize = sizeof(PSP_DEVICE_INTERFACE_DETAIL_DATA);
            // I get the following error
            // !! Cannot take the address or size of a variable of a managed type ('USBBoca.PSP_DEVICE_INTERFACE_DETAIL_DATA')
            // 

            // As a result.. this is hard coded for now !!! 
            // for the c struct PSP_DEVICE_INTERFACE_DETAIL_DATA dizeof = DWORD cbSize (size 4) + Char[0] (size 1) = Total size 5 ?
            //
            myPSP_DEVICE_INTERFACE_DETAIL_DATA.cbSize = 5;

            int results =
                SetupDiGetDeviceInterfaceDetail(
                hDevInfo,									// IN HDEVINFO  DeviceInfoSet,
                ref mySP_DEVICE_INTERFACE_DATA,				// IN PSP_DEVICE_INTERFACE_DATA  DeviceInterfaceData,
                ref myPSP_DEVICE_INTERFACE_DETAIL_DATA,		// DeviceInterfaceDetailData,  OPTIONAL
                DeviceInterfaceDetailDataSize,				// IN DWORD  DeviceInterfaceDetailDataSize,
                ref RequiredSize,							// OUT PDWORD  RequiredSize,  OPTIONAL
                null); // 
            DevicePathName = myPSP_DEVICE_INTERFACE_DETAIL_DATA.DevicePath;
            return results;
        }

        //-  CT_CreateFile(string DeviceName)
        public unsafe int CT_CreateFile(string DeviceName)
        {
            int status = 1;             //set status for success

            HidHandle = CreateFile(
                DeviceName,
                GENERIC_READ | GENERIC_WRITE,
                FILE_SHARE_READ | FILE_SHARE_WRITE,
                0,
                OPEN_EXISTING,
                0, 0);

            if (HidHandle == -1)
                status = 0;             //if handle bad, then status is failure

           return (status);
        }

        public unsafe int CT_CreateFile_Overlapped(string DeviceName)
        {

            HidHandle = CreateFile(
                DeviceName,
                GENERIC_READ | GENERIC_WRITE,
                FILE_SHARE_READ | FILE_SHARE_WRITE,
                ref security,
                OPEN_EXISTING,
                FILE_FLAG_OVERLAPED,
                0);                                 //@mdh overlapped I/O
            if (HidHandle == -1)
                return 0;
            else
                return 1;
        }
        //creates an event object for the ovelapped structure used with ReadFile
        public void prepareforoverlappedtransfer()
        {

            if (eventobject == 0)
            {
                eventobject = CreateEvent(ref security, 1, 1, "");
            }
            HIDOverlapped.OffsetLow = 0;
            HIDOverlapped.OffsetHigh = 0;
            HIDOverlapped.EventHandle = (System.IntPtr)eventobject;
        }

        public unsafe int CT_CloseHandle(int hObject)
        {
            HidHandle = -1;
            return CloseHandle(hObject);

        }

        //-  int CT_HidD_GetAttributes(int hObject))
        public unsafe int CT_HidD_GetAttributes(int hObject)
        {
            // Create an instance of HIDD_ATTRIBUTES
            myHIDD_ATTRIBUTES = new HIDD_ATTRIBUTES();
            // Calculate its size
            myHIDD_ATTRIBUTES.Size = sizeof(HIDD_ATTRIBUTES);

            return HidD_GetAttributes(
                    hObject,
                    ref myHIDD_ATTRIBUTES);
        }

        //HidD_GetSerialNumberString
        public unsafe bool CT_HidD_GetSerialNumberString(int HidDeviceObject, ref byte lpReportBuffer, int ReportBufferLength)
        {
            return HidD_GetSerialNumberString(HidDeviceObject, ref lpReportBuffer, ReportBufferLength);
        }

        //  int CT_HidD_GetPreparsedData(int hObject, ref int pPHIDP_PREPARSED_DATA)
        public unsafe int CT_HidD_GetPreparsedData(int hObject, ref int pPHIDP_PREPARSED_DATA)
        {
            return HidD_GetPreparsedData(
            hObject,
            ref pPHIDP_PREPARSED_DATA);
        }

        //  int CT_HidD_SetOutputReport(int HidDeviceObject, ref byte lpReportBuffer, int ReportBufferLength)
        public unsafe bool CT_HidD_SetOutputReport(int HidDeviceObject, ref byte lpReportBuffer, int ReportBufferLength)
        {
            return HidD_SetOutputReport(HidDeviceObject, ref lpReportBuffer, ReportBufferLength);
        }

        //  int CT_HidP_GetCaps(int pPreparsedData)
        public unsafe int CT_HidP_GetCaps(int pPreparsedData)
        {
            myHIDP_CAPS = new HIDP_CAPS();
            return HidP_GetCaps(
             pPreparsedData,
             ref myHIDP_CAPS);
        }

        public int CT_HidP_GetValueCaps(ref int CalsCapsLength, int pPHIDP_PREPARSED_DATA)
        {

            HIDP_REPORT_TYPE myType = 0;
            myHIDP_VALUE_CAPS = new HIDP_VALUE_CAPS[5];
            return HidP_GetValueCaps(
                myType,
                myHIDP_VALUE_CAPS,
                ref CalsCapsLength,
                pPHIDP_PREPARSED_DATA);

        }

        //-  byte[] CT_ReadFile(int InputReportByteLength)
        public unsafe byte[] CT_ReadFile(int InputReportByteLength)
        {
            int BytesRead = 0;
            int result = 0;
            bool success = false;
            uint count;
            byte[] BufBytes = new byte[InputReportByteLength];
            byte[] ptr_return = null;

            result = ReadFile(HidReadHandle, BufBytes, InputReportByteLength, ref BytesRead, ref HIDOverlapped);
            
            //read timeout .5 seconds
            result = WaitForSingleObject(HidReadHandle, boca_sample.FrmBoca.read_timeout);

            switch (result)
            {
                //completed
                case WAIT_OBJECT_0:
                    count = (uint)HIDOverlapped.InternalHigh;
                    success = GetOverlappedResult(HidReadHandle, ref HIDOverlapped, out count, true);
                    if (count > 0)
                        success = true;
                    byte[] OutBytes = new byte[BytesRead];
                    Array.Copy(BufBytes, OutBytes, BytesRead);
                    //return OutBytes;
                    ptr_return = OutBytes;

                    break;

                //timeout indicates the printer buffer is empty or the 
                //printer is not responding so cancel the IO operation
                case WAIT_TIMEOUT:
                    CancelIo(HidReadHandle);
                    //return null;
                    ptr_return = null;
                    break;

                default:
                    //return null;
                    ptr_return = null;
                    break;

            }
            
            return (ptr_return);

        }

        //  int CT_SetupDiDestroyDeviceInfoList()
        public int CT_SetupDiDestroyDeviceInfoList()
        {
            return SetupDiDestroyDeviceInfoList(hDevInfo);

        }

        //  int CT_HidD_FreePreparsedData(int pPHIDP_PREPARSED_DATA)
        public int CT_HidD_FreePreparsedData(int pPHIDP_PREPARSED_DATA)
        {
            return SetupDiDestroyDeviceInfoList(pPHIDP_PREPARSED_DATA);
        }

        internal USBHID.HidApiDeclarations HidApiDeclarations
        {
            get
            {
                throw new System.NotImplementedException();
            }
            set
            {
            }
        }




    }

    public class USBDevice
        {
            private string usbVID;
            private string usbPID;
            //private bool isConnected;

            //private HIDDevice usbdevice;

            //USB LIST BUFFER
            /// <summary>
            /// Buffer for incomming data.
            /// </summary>
            public static USBHIDDRIVER.List.ListWithEvent usbBuffer = new USBHIDDRIVER.List.ListWithEvent();
            public static byte[] aBuffer = new byte[1024];      //@mdh test
            //        public static string aBuffer;      //@mdh test

            /// <summary>
            /// Initializes a new instance of the <see cref="USBDevice"/> class.
            /// </summary>
            /// <param name="vid">The vendor id of the USB device (e.g. vid_06ba)</param>
            /// <param name="pid">The product id of the USB device (e.g. pid_ffff)</param>
            public USBDevice(string vid, string pid)
            {
                this.usbVID = vid;
                this.usbPID = pid;
                //this.usbdevice = new HIDDevice(this.usbVID, this.usbPID);
            }

            /// <summary>
            /// Initializes a new instance of the <see cref="USBDevice"/> class.
            /// </summary>
            /// <param name="vid">The vendor id of the USB device (e.g. vid_06ba).</param>
            public USBDevice(string vid)
            {
                this.usbVID = vid;
                //this.usbdevice = new HIDDevice(this.usbVID, "");
            }

            /// <summary>
            /// Establishes a connection to the USB device. 
            /// You can only establish a connection to a device if you have used the construct with vendor AND product id. 
            /// Otherwise it will connect to a device which has the same vendor id is specified, 
            /// this means if more than one device with these vendor id is plugged in, 
            /// you can't be determine to which one you will connect. 
            /// </summary>
            /// <returns>false if an error occures</returns>
            //public bool Connect()
            //{
                //isConnected = this.usbdevice.connectDevice();
                //return isConnected;
            //}

            /// <summary>
            /// Disconnects the device
            /// </summary>
            //public void Disconnect()
            //{
                //if (isConnected)
                //{
                    //this.usbdevice.disconnectDevice();
                //}
            //}

            /// <summary>
            /// Returns a list of devices with the vendor id (or vendor and product id) 
            /// specified in the constructor.
            /// This function is needed if you want to know how many (and which) devices with the specified
            /// vendor id are plugged in.
            /// </summary>
            /// <returns>String list with device paths</returns>
            //public List<string> getDeviceList()
           // {
                //return this.usbdevice.getDevices();
            //}
        /*
            /// <summary>
            /// Writes the specified bytes to the USB device.
            /// If the array length exceeds 64, the array while be divided into several 
            /// arrays with each containing 64 bytes.
            /// The 0-63 byte of the array is sent first, then the 64-127 byte and so on.
            /// </summary>
            /// <param name="bytes">The bytes to send.</param>
            /// <returns>Returns true if all bytes have been written successfully</returns>
            public bool write(Byte[] bytes)
            {
                int byteCount = bytes.Length;
                int bytePos = 0;

                bool success = true;

                //build hid reports with 64 bytes
                while (bytePos <= byteCount - 1)
                {
                    if (bytePos > 0)
                    {
                        Thread.Sleep(5);
                    }
                    Byte[] transfByte = new byte[64];
                    for (int u = 0; u < 64; u++)
                    {
                        if (bytePos < byteCount)
                        {
                            transfByte[u] = bytes[bytePos];
                            bytePos++;
                        }
                        else
                        {
                            transfByte[u] = 0;
                        }
                    }
                    //send the report
                    if (!this.usbdevice.writeData(transfByte))
                    {
                        success = false;
                    }
                    Thread.Sleep(5);
                }
                return success;
            }
        */
            /// <summary>
            /// Starts reading. 
            /// If you execute this command a thread is started which listens to the USB device and waits for data.
            /// </summary>
            //public void startRead()
            //{
                //this.usbdevice.readData();
            //}

            /// <summary>
            /// Stops the read thread.
            /// By executing this command the read data thread is stopped and now data will be received.
            /// </summary>
            //public void stopRead()
            //{
                //this.usbdevice.readData();
            //}

            /// <summary>
            /// Enables the usb buffer event.
            /// Whenever a dataset is added to the buffer (and so received from the usb device)
            /// the event handler method will be called.
            /// </summary>
            /// <param name="eHandler">The event handler method.</param>
            public void enableUsbBufferEvent(System.EventHandler eHandler)
            {
                usbBuffer.Changed += eHandler;
            }
        }

    sealed class HidApiDeclarations
        {

            // API Declarations for communicating with HID-class devices.
            // ******************************************************************************
            // API constants
            // ******************************************************************************
            // from hidpi.h
            // Typedef enum defines a set of integer constants for HidP_Report_Type
            public const short HidP_Input = 0;
            public const short HidP_Output = 1;
            public const short HidP_Feature = 2;


            // ******************************************************************************
            // Structures and classes for API calls, listed alphabetically
            // ******************************************************************************

            [StructLayout(LayoutKind.Sequential)]
            public struct HIDD_ATTRIBUTES
            {
                public int Size;
                public short VendorID;
                public short ProductID;
                public short VersionNumber;
            }

            [StructLayout(LayoutKind.Sequential)]
            public struct HIDP_CAPS
            {
                public short Usage;
                public short UsagePage;
                public short InputReportByteLength;
                public short OutputReportByteLength;
                public short FeatureReportByteLength;
                [MarshalAs(UnmanagedType.ByValArray, SizeConst = 17)]
                public short[] Reserved;
                public short NumberLinkCollectionNodes;
                public short NumberInputButtonCaps;
                public short NumberInputValueCaps;
                public short NumberInputDataIndices;
                public short NumberOutputButtonCaps;
                public short NumberOutputValueCaps;
                public short NumberOutputDataIndices;
                public short NumberFeatureButtonCaps;
                public short NumberFeatureValueCaps;
                public short NumberFeatureDataIndices;

            }

            // If IsRange is false, UsageMin is the Usage and UsageMax is unused.
            // If IsStringRange is false, StringMin is the string index and StringMax is unused.
            // If IsDesignatorRange is false, DesignatorMin is the designator index and DesignatorMax is unused.

            [StructLayout(LayoutKind.Sequential)]
            public struct HidP_Value_Caps
            {
                public short UsagePage;
                public byte ReportID;
                public int IsAlias;
                public short BitField;
                public short LinkCollection;
                public short LinkUsage;
                public short LinkUsagePage;
                public int IsRange;
                public int IsStringRange;
                public int IsDesignatorRange;
                public int IsAbsolute;
                public int HasNull;
                public byte Reserved;
                public short BitSize;
                public short ReportCount;
                public short Reserved2;
                public short Reserved3;
                public short Reserved4;
                public short Reserved5;
                public short Reserved6;
                public int LogicalMin;
                public int LogicalMax;
                public int PhysicalMin;
                public int PhysicalMax;
                public short UsageMin;
                public short UsageMax;
                public short StringMin;
                public short StringMax;
                public short DesignatorMin;
                public short DesignatorMax;
                public short DataIndexMin;
                public short DataIndexMax;
            }

            // ******************************************************************************
            // API functions, listed alphabetically
            // ******************************************************************************

            [DllImport("hid.dll")]
            static public extern bool HidD_FlushQueue(int HidDeviceObject);

            [DllImport("hid.dll")]
            static public extern bool HidD_FreePreparsedData(ref IntPtr PreparsedData);

            [DllImport("hid.dll")]
            static public extern int HidD_GetAttributes(int HidDeviceObject, ref HIDD_ATTRIBUTES Attributes);

            [DllImport("hid.dll")]
            static public extern bool HidD_GetFeature(int HidDeviceObject, ref byte lpReportBuffer, int ReportBufferLength);

            [DllImport("hid.dll")]
            static public extern bool HidD_GetInputReport(int HidDeviceObject, ref byte lpReportBuffer, int ReportBufferLength);

            [DllImport("hid.dll")]
            static public extern void HidD_GetHidGuid(ref System.Guid HidGuid);

            [DllImport("hid.dll")]
            static public extern bool HidD_GetNumInputBuffers(int HidDeviceObject, ref int NumberBuffers);

            [DllImport("hid.dll")]
            static public extern bool HidD_GetPreparsedData(int HidDeviceObject, ref IntPtr PreparsedData);

            [DllImport("hid.dll")]
            static public extern bool HidD_SetFeature(int HidDeviceObject, ref byte lpReportBuffer, int ReportBufferLength);

            [DllImport("hid.dll")]
            static public extern bool HidD_SetNumInputBuffers(int HidDeviceObject, int NumberBuffers);

            [DllImport("hid.dll")]
            static public extern bool HidD_SetOutputReport(int HidDeviceObject, ref byte lpReportBuffer, int ReportBufferLength);

            [DllImport("hid.dll")]
            static public extern int HidP_GetCaps(IntPtr PreparsedData, ref HIDP_CAPS Capabilities);

            [DllImport("hid.dll")]
            static public extern int HidP_GetValueCaps(short ReportType, ref byte ValueCaps, ref short ValueCapsLength, IntPtr PreparsedData);

            [DllImport("hid.dll")]
            static public extern bool HidD_GetSerialNumberString(int HidDeviceObject, ref byte lpReportBuffer, int ReportBufferLength);

        }

}


